@@ -16,4 +16,12 @@ class InvalidCookies(Exception):
 
 class LoginRequired(Exception):
     '''Facebook requires a login to see this'''
+    pass
+
+class LoginError(Exception):
+    '''Failed to log in'''
+    pass
+
+class UnexpectedResponse(Exception):
+    '''Facebook served something weird'''
     pass
\ No newline at end of file
