@@ -27,21 +27,20 @@ logger = logging.getLogger(__name__)
 PartialPost = Optional[Dict[str, Any]]
 
 
-def extract_post(raw_post: RawPost, options: Options, request_fn: RequestFunction) -> Post:
-    return PostExtractor(raw_post, options, request_fn).extract_post()
+def extract_post(raw_post: RawPost, options: Options, request_fn: RequestFunction, full_post_html = None) -> Post:
+    return PostExtractor(raw_post, options, request_fn, full_post_html).extract_post()
 
+def extract_group_post(raw_post: RawPost, options: Options, request_fn: RequestFunction, full_post_html = None) -> Post:
+    return GroupPostExtractor(raw_post, options, request_fn, full_post_html).extract_post()
 
-def extract_group_post(raw_post: RawPost, options: Options, request_fn: RequestFunction) -> Post:
-    return GroupPostExtractor(raw_post, options, request_fn).extract_post()
-
-def extract_photo_post(raw_post: RawPost, options: Options, request_fn: RequestFunction) -> Post:
-    return PhotoPostExtractor(raw_post, options, request_fn).extract_post()
+def extract_photo_post(raw_post: RawPost, options: Options, request_fn: RequestFunction, full_post_html) -> Post:
+    return PhotoPostExtractor(raw_post, options, request_fn, full_post_html).extract_post()
 
 
 class PostExtractor:
     """Class for Extracting fields from a FacebookPost"""
 
-    likes_regex = re.compile(r'([\d.KM]+)\s+(Like|left reaction)', re.IGNORECASE)
+    likes_regex = re.compile(r'([\d,.KM]+)\s+(Like|left reaction)', re.IGNORECASE)
     comments_regex = re.compile(r'([\d,.KM]+)\s+comment', re.IGNORECASE)
     shares_regex = re.compile(r'([\d,.KM]+)\s+Share', re.IGNORECASE)
     live_regex = re.compile(r'.+(is live).+')
@@ -50,7 +49,7 @@ class PostExtractor:
     photo_link = re.compile(r'href=\"(/[^\"]+/photos/[^\"]+?)\"')
     photo_link_2 = re.compile(r'href=\"(/photo.php[^\"]+?)\"')
     image_regex = re.compile(
-        r'<a href=\"([^\"]+?)\" target=\"_blank\" class=\"sec\">View Full Size<\/a>',
+        r'<a href=\"([^\"]+?)\" target=\"_blank\" class=\"sec\">',
         re.IGNORECASE,
     )
     image_regex_lq = re.compile(r"background-image: url\('(.+)'\)")
@@ -68,13 +67,13 @@ class PostExtractor:
     more_url_regex = re.compile(r'(?<=…\s)<a href="([^"]+)')
     post_story_regex = re.compile(r'href="(\/story[^"]+)" aria')
 
-    def __init__(self, element, options, request_fn):
+    def __init__(self, element, options, request_fn, full_post_html = None):
         self.element = element
         self.options = options
         self.request = request_fn
 
         self._data_ft = None
-        self._full_post_html = None
+        self._full_post_html = full_post_html
         self._live_data = {}
 
     # TODO: This is getting ugly, create a dataclass for Post
@@ -120,6 +119,8 @@ class PostExtractor:
             'comments_full': None,
             'reactors': None,
             'w3_fb_url': None,
+            'reactions': None,
+            'reaction_count': None,
         }
 
     def extract_post(self) -> Post:
@@ -146,6 +147,7 @@ class PostExtractor:
             self.extract_factcheck,
             self.extract_share_information,
             self.extract_availability,
+            self.extract_listing
         ]
 
         post = self.make_new_post()
@@ -217,7 +219,7 @@ class PostExtractor:
             element = self.full_post_html.find('.story_body_container', first=True)
 
         nodes = element.find('p, header, span[role=presentation]')
-        if nodes:
+        if nodes and len(nodes) > 1:
             post_text = []
             shared_text = []
             ended = False
@@ -250,6 +252,18 @@ class PostExtractor:
                 'post_text': post_text,
                 'shared_text': shared_text,
             }
+        elif element.find(".story_body_container>div", first=True):
+            text = element.find(".story_body_container>div", first=True).text
+            return {
+                'text': text,
+                'post_text': text
+            }
+        elif len(nodes) == 1:
+            text = nodes[0].text
+            return {
+                'text': text,
+                'post_text': text
+            }
 
         return None
 
@@ -412,10 +426,16 @@ class PostExtractor:
             return None
         images = []
         descriptions = []
-        photo_links = self.element.find("div.story_body_container>div a[href*='photo.php'], div.story_body_container>div a[href*='/photos/']")
+        image_ids = []
+        photo_links = self.element.find(
+            "div.story_body_container>div a[href*='photo.php'], "
+            "div.story_body_container>div a[href*='/photos/'], "
+            "div._5v64 a[href*='/photos/']"
+        )
+
         total_photos_in_gallery = len(photo_links)
-        if len(photo_links) == 4 and photo_links[-1].text:
-            total_photos_in_gallery = 4 + int(photo_links[-1].text.strip("+")) - 1
+        if len(photo_links) in [4,5] and photo_links[-1].text:
+            total_photos_in_gallery = len(photo_links) + int(photo_links[-1].text.strip("+")) - 1
             logger.debug(f"{total_photos_in_gallery} total photos in gallery")
 
         # This gets up to 4 images in gallery
@@ -423,29 +443,66 @@ class PostExtractor:
             url = link.attrs["href"]
             if "photoset_token" in url:
                 query = parse_qs(urlparse(url).query)
-                photo_id = query["photo"][0]
                 profile_id = query["profileid"][0]
-                url = f"/photo.php?fbid={photo_id}&profileid={profile_id}"
-
+                token = query["photoset_token"][0]
+                url = f"{profile_id}/posts/{token}"
+                logger.debug(f"Fetching {url}")
+                response = self.request(url)
+                results = self.get_jsmod("mtouch_snowflake_paged_query", response.html)
+                results = list(results["query_results"].values())[0]["media"]
+                video_ids = []
+                videos = []
+                for item in results["edges"]:
+                    node = item["node"]
+                    if node["is_playable"]:
+                        video_ids.append(node["id"])
+                        videos.append(node["playable_url_hd"] or node["playable_url"])
+                    images.append(node["full_width_image"]["uri"])
+                    image_ids.append(node["id"])
+                    descriptions.append(node["accessibility_caption"])
+                return {
+                    "image": images[0] if images else None,
+                    "images": images,
+                    "images_description": descriptions,
+                    "image_id": image_ids[0] if image_ids else None,
+                    "image_ids": image_ids,
+                    "video": videos[0] if videos else None,
+                    "video_id": video_ids[0] if video_ids else None,
+                    "video_ids": video_ids,
+                    "videos": videos
+                }
             url = utils.urljoin(FB_MOBILE_BASE_URL, url)
             logger.debug(f"Fetching {url}")
-            response = self.request(url)
-            images.append(self.extract_photo_link_HQ(response.text))
-            elem = response.html.find(".img[data-sigil='photo-image']", first=True)
-            descriptions.append(elem.attrs.get("alt") or elem.attrs.get("aria-label"))
+            try:
+                response = self.request(url)
+                images.append(self.extract_photo_link_HQ(response.text))
+                elem = response.html.find(".img[data-sigil='photo-image']", first=True)
+                descriptions.append(elem.attrs.get("alt") or elem.attrs.get("aria-label"))
+                image_ids.append(re.search(r'[=/](\d+)', url).group(1))
+            except Exception as e:
+                logger.error(e)
+                total_photos_in_gallery -= 1
 
         while len(images) < total_photos_in_gallery:
             # More photos to fetch. Follow the left arrow link of the last image we were on
-            url = response.html.find('a.touchable[data-gt=\'{"tn":"+>"}\']', first=True).attrs["href"]
+            direction = '{"tn":"+>"}'
+            if response.html.find("a", containing="Photos from", first=True):
+                # Right arrow link
+                direction = '{"tn":"+="}'
+            url = response.html.find(f"a.touchable[data-gt='{direction}']", first=True).attrs["href"]
             if not url.startswith("http"):
                 url = utils.urljoin(FB_MOBILE_BASE_URL, url)
             logger.debug(f"Fetching {url}")
             response = self.request(url)
-            images.append(self.extract_photo_link_HQ(response.text))
-            elem = response.html.find(".img[data-sigil='photo-image']", first=True)
-            descriptions.append(elem.attrs.get("alt") or elem.attrs.get("aria-label"))
+            photo_link = self.extract_photo_link_HQ(response.text)
+            if photo_link not in images:
+                images.append(photo_link)
+                elem = response.html.find(".img[data-sigil='photo-image']", first=True)
+                descriptions.append(elem.attrs.get("alt") or elem.attrs.get("aria-label"))
+                image_ids.append(re.search(r'[=/](\d+)', url).group(1))
         image = images[0] if images else None
-        return {"image": image, "images": images, "images_description": descriptions}
+        image_id = image_ids[0] if image_ids else None
+        return {"image": image, "images": images, "images_description": descriptions, "image_id": image_id, "image_ids": image_ids}
 
     def extract_reactions(self) -> PartialPost:
         """Fetch share and reactions information with a existing post obtained by `get_posts`.
@@ -525,6 +582,7 @@ class PostExtractor:
             return {
                 'likes': reactions.get("like"),
                 'reactions': reactions,
+                'reaction_count': self.live_data.get("reactioncount"),
                 'reactors': reactors,
                 'fetched_time': datetime.now(),
                 'w3_fb_url': w3_fb_url,
@@ -552,6 +610,21 @@ class PostExtractor:
 
     def extract_video(self):
         video_data_element = self.element.find('[data-sigil="inlineVideo"]', first=True)
+        photoset_link = self.element.find("a[href*='photoset_token']", first=True)
+        if photoset_link and photoset_link.find("i[aria-label='video']"):
+            query = parse_qs(urlparse(photoset_link.attrs.get("href")).query)
+            video_id = query["photo"][0]
+            logger.debug(f"Fetching {video_id}")
+            response = self.request(video_id)
+            video_post = PostExtractor(response.html, self.options, self.request, full_post_html=response.html)
+            video_post.post = {"post_id": video_id}
+            meta = video_post.extract_video_meta() or {}
+            return {
+                "video_id": video_id,
+                "video": video_post.extract_video().get("video"),
+                **meta
+            }
+
         if video_data_element is None:
             return None
         if self.options.get('youtube_dl'):
@@ -584,13 +657,8 @@ class PostExtractor:
             ydl_opts['quiet'] = False
 
         try:
-            post_id = self.post.get('post_id')
-            if post_id is None:
-                return None
-
-            video_page = 'https://www.facebook.com/' + post_id
             with YoutubeDL(ydl_opts) as ydl:
-                url = ydl.extract_info(video_page, download=False)['url']
+                url = ydl.extract_info(self.post.get("post_url"), download=False)['url']
                 return {'video': url}
         except ExtractorError as ex:
             logger.error("Error extracting video with youtube-dl: %r", ex)
@@ -712,6 +780,10 @@ class PostExtractor:
         else:
             date = None
 
+        image_url = comment.find('[data-sigil="comment-body"] + div a[href]', first=True)
+        if image_url and image_url.attrs["href"].startswith("https://lm.facebook.com/l.php"):
+            image_url = parse_qs(urlparse(image_url.attrs["href"]).query).get("u")
+
         return {
             "comment_id": comment_id,
             "comment_url": utils.urljoin(FB_BASE_URL, comment_id),
@@ -720,6 +792,7 @@ class PostExtractor:
             "commenter_meta": commenter_meta,
             "comment_text": text,
             "comment_time": date,
+            "comment_image": image_url
         }
 
     def extract_comments_full(self):
@@ -731,11 +804,12 @@ class PostExtractor:
             return
         comments = list(elem.find('div[data-sigil="comment"]'))
 
+        more_selector = f"a[href*='{self.post.get('post_id')}']"
         direction = "View more comments"
-        more = elem.find("a", containing=direction, first=True)
+        more = elem.find(more_selector, containing=direction, first=True)
         if not more:
             direction = "View previous comments"
-            more = elem.find("a", containing=direction, first=True)
+            more = elem.find(more_selector, containing=direction, first=True)
 
         # Comment limiting and progress
         limit = 5000 # Default
@@ -774,7 +848,7 @@ class PostExtractor:
                 break
             more_comments = elem.find('div[data-sigil="comment"]')
             comments.extend(more_comments)
-            more = elem.find("a", containing=direction, first=True)
+            more = elem.find(more_selector, containing=direction, first=True)
 
         logger.debug(f"Found {len(comments)} comments")
         results = []
@@ -806,6 +880,16 @@ class PostExtractor:
             good_json = self.bad_json_key_regex.sub(r'\g<prefix>"\g<key>":', bad_json)
             yield json.loads(good_json)
 
+    def extract_listing(self) -> PartialPost:
+        # Marketplace listings
+        divs = self.element.find("div[data-ft='{\"tn\":\"H\"}']>div>div")
+        if len(divs) >= 3:
+            return {
+                "listing_title": divs[0].find("span")[-1].text,
+                "listing_price": divs[1].text,
+                "listing_location": divs[2].text
+            }
+
     @property
     def data_ft(self) -> dict:
         if self._data_ft is not None:
@@ -827,7 +911,7 @@ class PostExtractor:
         if self._full_post_html is not None:
             return self._full_post_html
 
-        if self.options.get("allow_extra_requests", True):
+        if self.options.get("allow_extra_requests", True) and self.post.get('post_url'):
             url = self.post.get('post_url').replace(FB_BASE_URL, FB_MOBILE_BASE_URL)
             logger.debug(f"Fetching {url}")
             response = self.request(url)
@@ -846,11 +930,18 @@ class PostExtractor:
             logger.error(e)
         return self._live_data
 
-    def get_jsmod(self, name):
-        match = re.search(name + r'[^{]+({.+?})(?:\]\]|,\d)', self.full_post_html.html)
+    def get_jsmod(self, name, element = None):
+        if not element:
+            if self.full_post_html:
+                element = self.full_post_html
+            else:
+                element = self.element
+        match = re.search(name + r'[^{]+({.+?})(?:\]\]|,\d)', element.html)
         if match:
             # Use demjson to load JS, as unquoted keys is not valid JSON
             return demjson.decode(match.group(1))
+        else:
+            return {}
 
 
 class GroupPostExtractor(PostExtractor):
@@ -877,9 +968,3 @@ class PhotoPostExtractor(PostExtractor):
 
     def extract_post_id(self) -> PartialPost:
         return {"post_id": str(self.live_data["ft_ent_identifier"])}
-
-    def extract_likes(self) -> PartialPost:
-        return {"likes": self.live_data["like_count"]}
-
-    def extract_comments(self) -> PartialPost:
-        return {"comments": self.live_data["comment_count"]}
\ No newline at end of file
