@@ -13,7 +13,7 @@ from . import utils
 from .constants import DEFAULT_PAGE_LIMIT, FB_BASE_URL, FB_MOBILE_BASE_URL, FB_W3_BASE_URL
 from .extractors import extract_group_post, extract_post, extract_photo_post
 from .fb_types import Post, Profile
-from .page_iterators import iter_group_pages, iter_pages
+from .page_iterators import iter_group_pages, iter_pages, iter_photos
 from . import exceptions
 
 
@@ -46,6 +46,15 @@ class FacebookScraper:
         self.session = session
         self.requests_kwargs = requests_kwargs
 
+    def set_user_agent(self, user_agent):
+        self.session.headers["User-Agent"] = user_agent
+
+    def set_noscript(self, noscript):
+        if noscript:
+            self.session.cookies.set("noscript", "1")
+        else:
+            self.session.cookies.set("noscript", "0")
+
     def set_proxy(self, proxy):
         self.requests_kwargs.update({
             'proxies': {
@@ -53,13 +62,18 @@ class FacebookScraper:
                 'https': proxy
             }
         })
-        ip = self.get("http://ifconfig.co", headers={"Accept": "application/json"}).json()
+        ip = self.get("http://ipinfo.io", headers={"Accept": "application/json"}).json()
         logger.debug(f"Proxy details: {ip}")
 
     def get_posts(self, account: str, **kwargs) -> Iterator[Post]:
+        kwargs["scraper"] = self
         iter_pages_fn = partial(iter_pages, account=account, request_fn=self.get, **kwargs)
         return self._generic_get_posts(extract_post, iter_pages_fn, **kwargs)
 
+    def get_photos(self, account: str, **kwargs) -> Iterator[Post]:
+        iter_pages_fn = partial(iter_photos, account=account, request_fn=self.get, **kwargs)
+        return self._generic_get_posts(extract_post, iter_pages_fn, **kwargs)
+
     def get_posts_by_url(self, post_urls, options={}, remove_source=True) -> Iterator[Post]:
         for post_url in post_urls:
             url = str(post_url)
@@ -75,7 +89,7 @@ class FacebookScraper:
             }
             logger.debug(f"Requesting page from: {url}")
             response = self.get(url)
-            elem = response.html.find('article[data-ft],div.async_like[data-ft]', first=True)
+            elem = response.html.find('[data-ft*="top_level_post_id"]', first=True)
             photo_post = False
             if response.html.find("div.msg", first=True):
                 photo_post = True
@@ -89,11 +103,11 @@ class FacebookScraper:
                     elem = utils.make_html_element(elem.html.replace("</footer>", comments_area.html + "</footer>"))
 
                 if photo_post:
-                    post.update(extract_photo_post(elem, request_fn=self.get, options=options))
+                    post.update(extract_photo_post(elem, request_fn=self.get, options=options, full_post_html=response.html))
                 elif url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "/groups/")):
-                    post.update(extract_group_post(elem, request_fn=self.get, options=options))
+                    post.update(extract_group_post(elem, request_fn=self.get, options=options, full_post_html=response.html))
                 else:
-                    post.update(extract_post(elem, request_fn=self.get, options=options))
+                    post.update(extract_post(elem, request_fn=self.get, options=options, full_post_html=response.html))
                 if not post.get("post_url"):
                     post["post_url"] = url
                 if remove_source:
@@ -216,8 +230,21 @@ class FacebookScraper:
         try:
             resp = self.get(url).html
             admins = resp.find("div:first-child>div.touchable a:not(.touchable)")
-            result["admins"] = [{"name": e.text, "link": e.attrs["href"]} for e in admins]
-        except Exception as e:
+            result["admins"] = [{"name": e.text, "link": e.attrs["href"].split("?")[0]} for e in admins]
+            url = resp.find("a[href^='/browse/group/members']", first=True).attrs["href"]
+            members = []
+            while url:
+                logger.debug(f"Requesting page from: {url}")
+                resp = self.get(url).html
+                elems = resp.find("#root div.touchable a:not(.touchable)")
+                members.extend([{"name": e.text, "link": e.attrs["href"]} for e in elems])
+                more = re.search(r'"m_more_item",href:"([^"]+)', resp.text)
+                if more:
+                    url = more.group(1)
+                else:
+                    url = None
+            result["members"] = [m for m in members if m not in result["admins"]]
+        except exceptions.LoginRequired as e:
             pass
         return result
 
@@ -243,6 +270,8 @@ class FacebookScraper:
             response.html.html = response.html.html.replace('<!--', '').replace('-->', '')
             response.raise_for_status()
             self.check_locale(response)
+            if "noscript" not in response.html.html:
+                warnings.warn(f"Facebook served mbasic/noscript content unexpectedly on {response.url}")
             title = response.html.find("title", first=True)
             not_found_titles = ["page not found", "content not found"]
             temp_ban_titles = ["you can't use this feature at the moment", "you’re temporarily blocked"]
@@ -253,35 +282,53 @@ class FacebookScraper:
                     raise exceptions.TemporarilyBanned(title.text)
                 elif ">Your Account Has Been Disabled<" in response.html.html:
                     raise exceptions.AccountDisabled("Your Account Has Been Disabled")
-                elif title.text == "Log in to Facebook | Facebook" or response.url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "login")):
+                elif (title.text == "Log in to Facebook | Facebook" or
+                      response.url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "login")) or
+                      response.url.startswith(utils.urljoin(FB_W3_BASE_URL, "login")) or
+                      (", log in to Facebook." in response.text and not response.html.find("article[data-ft],div.async_like[data-ft],div.msg"))
+                      ):
                     raise exceptions.LoginRequired("A login (cookies) is required to see this page")
             return response
         except RequestException as ex:
             logger.exception("Exception while requesting URL: %s\nException: %r", url, ex)
             raise
 
-    def login(self, email: str, password: str):
-        login_page = self.get(self.base_url)
-        login_action = login_page.html.find('#login_form', first=True).attrs.get('action')
-
-        elems = login_page.html.find('#login_form > input[name][value]')
+    def submit_form(self, response, extra_data = {}):
+        action = response.html.find("form", first=True).attrs.get('action')
+        url = utils.urljoin(self.base_url, action)
+        elems = response.html.find("input[name][value]")
         data = { elem.attrs['name']: elem.attrs['value'] for elem in elems }
-        data["email"] = email
-        data["pass"] = password
+        data.update(extra_data)
+        response = self.session.post(url, data=data)
+        return response
 
-        response = self.session.post(
-            utils.urljoin(self.base_url, login_action), data=data
-        )
-        response_text = response.html.find('#viewport', first=True).text
-
-        logger.debug("Login response text: %s", response_text)
+    def login(self, email: str, password: str):
+        response = self.get(self.base_url)
+        response = self.submit_form(response, {"email": email, "pass": password, "_fb_noscript": None})
 
         login_error = response.html.find('#login_error', first=True)
         if login_error:
-            logger.error("Login error: %s", login_error.text)
+            raise exceptions.LoginError(login_error.text)
 
+        if "Enter login code to continue" in response.text:
+            token = input("Enter 2FA token: ")
+            response = self.submit_form(response, {"approvals_code": token})
+            strong = response.html.find("strong", first=True)
+            if strong and strong.text.startswith("The login code you entered doesn't match"):
+                raise exceptions.LoginError(strong.text)
+            # Remember Browser
+            response = self.submit_form(response, {"name_action_selected": "save_device"})
+            if "Review recent login" in response.text:
+                response = self.submit_form(response)
+                # Login near {location} from {browser} on {OS} ({time}). Unset "This wasn't me", leaving "This was me" set.
+                response = self.submit_form(response, {"submit[This wasn't me]": None})
+                # Remember Browser. Please save the browser that you just verified. You won't have to enter a code when you log in from browsers that you've saved.
+                response = self.submit_form(response, {"name_action_selected": "save_device"})
+
+        if "Login approval needed" in response.text or "checkpoint" in response.url:
+            raise exceptions.LoginError("Login approval needed")
         if 'c_user' not in self.session.cookies:
-            warnings.warn('login unsuccessful')
+            raise exceptions.LoginError("Login unsuccessful")
 
     def is_logged_in(self) -> bool:
         try:
@@ -297,6 +344,7 @@ class FacebookScraper:
         page_limit=DEFAULT_PAGE_LIMIT,
         options=None,
         remove_source=True,
+        **kwargs
     ):
         counter = itertools.count(0) if page_limit is None else range(page_limit)
 
